/***************************************************************************
 *   Copyright (C) 2004 by Maciej "Kasprzol" Kasprzyk                      *
 *   katalogue@katalogue.szluug.org                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "findfiledialog.h"
#include "mydirinfo.h"
#include "treeview.h"

#include <qregexp.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qlabel.h>

#include <klocale.h>
#include <kguiitem.h>

#include <iostream>

using std::vector ;

FindFileDialog::FindFileDialog(QWidget *parent, MyKatalog * myk)
 : KDialogBase (KDialogBase::Plain, i18n ("Find File"), KDialogBase::Ok|KDialogBase::Close, KDialogBase::Ok, parent, "findfile", true, true),
   m_katalog (myk)
{

	QFrame * page = plainPage () ;
	QGridLayout * grid = new QGridLayout ( page, 5, 1, 3 /* space beetwen elements in pixels */ );
	
	label1 = new QLabel ( i18n ("filename (may include wildcards - '?' & '*'):"), page, "filename_label" ) ;
	grid->addWidget (label1, 0, 0 ) ;
	m_le_regexp = new QLineEdit( page, "le_regexp" ) ;
	grid->addWidget (m_le_regexp, 1, 0 ) ;
	m_ch_perl = new QCheckBox (i18n ("Treat this filename as Perl-compatible regexp"), page, "perl_checkbox") ;
	grid->addWidget (m_ch_perl, 2, 0 ) ;
	
	
	m_treeview = new TreeView ( page, "fileview" ) ;
	grid->addWidget (m_treeview, 3, 0) ;
	m_treeview->addColumn ( "Name" ) ;
	m_treeview->addColumn ( "Size" ) ;
	m_treeview->addColumn ( "Date" ) ;
	m_treeview->addColumn ( "Mode" ) ;
	m_treeview->addColumn ( "uid/gid" ) ;
	m_treeview->addColumn ( "type" ) ;
	m_treeview->addColumn ( "full path" ) ;
	m_treeview->setAllColumnsShowFocus (true) ;
//	fileview->setSorting (-1) ;
	m_treeview->setShowToolTips (true) ;
	m_treeview->setShowSortIndicator (true) ;
	
	label2 = new QLabel ( i18n ("Found <b>0</b> items."), page, "found_label" ) ;
	grid->addWidget (label2, 4, 0) ;
	
	setButtonOK ( KGuiItem ( i18n ("Search"), QString::null, i18n ("Search for files that match the given regexp"), i18n ("Search for files that match the given regexp") ) ) ;
}

FindFileDialog::~FindFileDialog()
{
	delete m_le_regexp ;
	delete m_ch_perl ;
}

void FindFileDialog::SearchDirForRegexp (const MyDirInfo * mdi, const QRegExp & regexp)
{
	vector<MyDirInfo*>::const_iterator mdiit ;
	vector<MyFileInfo*>::const_iterator mfi_cit ;
	
	// najpierw sprawd czy mdi spenia kryteria wyszukiwania.
	if (regexp.search ( mdi->name.c_str () ) != -1) {
		new TreeItem (m_treeview, const_cast<MyDirInfo*>(mdi)) ; // TreeItem'y s niszczone przez list podczas zmiany zawartoci
	}
	// nastpnie podkatalogii.
	for (mdiit = mdi->subdirs.begin () ; mdiit != mdi->subdirs.end () ; ++mdiit ) {
		SearchDirForRegexp (*mdiit, regexp) ; // tutaj nie trzeba sprawdza czy mdiit pasuje do regexpa poniewa
						// zostanie to zrobione na pocztku wywoania SearchDirForRegexp (*mdiit)
	}
	// a na kocu pliki w mdi.
	for (mfi_cit = mdi->files.begin () ; mfi_cit != mdi->files.end () ; ++mfi_cit) {
		if (regexp.search ( (*mfi_cit)->name.c_str () ) != -1) {
			new TreeItem (m_treeview, const_cast<MyFileInfo*>(*mfi_cit)) ; // TreeItem'y s niszczone przez list podczas zmiany zawartoci
		}
	}
}

/** No descriptions */
void FindFileDialog::slotOk()
{
/*
QRegExp rx1( "^\\d\\d?$" );  // match integers 0 to 99
    rx1.search( "123" );         // returns -1 (no match)
    rx1.search( "-6" );          // returns -1 (no match)
    rx1.search( "6" );           // returns 0 (matched as position 0)
//The third string matches '6'. This is a simple validation regexp for integers in the range 0 to 99. 
    QRegExp rx2( "^\\S+$" );     // match strings without whitespace
    rx2.search( "Hello world" ); // returns -1 (no match)
    rx2.search( "This_is-OK" );  // returns 0 (matched at position 0)
   QRegExp rx3( "*.html" );         // invalid regexp: * doesn't quantify anything
    rx3.setWildcard( TRUE );         // now it's a valid wildcard regexp
    rx3.exactMatch( "index.html" );  // returns TRUE
    rx3.exactMatch( "default.htm" ); // returns FALSE
    rx3.exactMatch( "readme.txt" );  // returns FALSE 
*/
	QRegExp regexp ( m_le_regexp->text () ) ;
	vector <MyDiskGroup*>::const_iterator mdg_cit ;
	vector <MyTopLevelDirInfo*>::const_iterator mdi_cit ;
	if ( ! m_ch_perl->isChecked () ) {
		regexp.setWildcard ( true ) ;
	}
	m_treeview->clear () ;
	for (mdg_cit = m_katalog->DiskGroups.begin () ; mdg_cit != m_katalog->DiskGroups.end () ; ++mdg_cit) {
		for (mdi_cit = (*mdg_cit)->Disks.begin () ; mdi_cit != (*mdg_cit)->Disks.end () ; ++mdi_cit) {
			SearchDirForRegexp (*mdi_cit, regexp) ;
		}
	}
	for (mdi_cit = m_katalog->Disks.begin () ; mdi_cit != m_katalog->Disks.end () ; ++mdi_cit) {
		SearchDirForRegexp (*mdi_cit, regexp) ;
	}
	label2->setText ( i18n ("Found <b>") + QString ("%1").arg (m_treeview->childCount ()) + i18n ("</b> items.") ) ;
	
//	accept () ;
}
