/***************************************************************************
 *   Copyright (C) 2004 by Maciej "Kasprzol" Kasprzyk                      *
 *   katalogue@katalogue.szluug.org                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "katalogue.h"
#include "katalogueview.h"
#include "pref.h"
#include "scandirdialog.h"
#include "findfiledialog.h"
#include "mydirinfo.h"
#include "mydocument.h"
#include "varibles.h"
#include <iostream>

#include <qdragobject.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>

#include <kglobal.h>
#include <klocale.h>
//#include <kiconloader.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kkeydialog.h>
#include <kaccel.h>
//#include <kio/netaccess.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kurl.h>
//#include <kurlrequesterdlg.h>
//#include <kprinter.h>
#include <kapplication.h>

#include <kedittoolbar.h>

#include <kmessagebox.h>
#include <kinputdialog.h>
#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>

//#include <kstandarddirs.h>
#include <kiconloader.h> // adowanie standardowych ikon

Katalogue::Katalogue()
    : KMainWindow( 0, "Katalogue" ),
      m_mydoc (new MyDocument),
      m_view(new KatalogueView (this, m_mydoc))/*,
      m_printer(0)*/
{
    // set the shell's ui resource file
    // setXMLFile("katalogueui.rc");
    
    // accept dnd
    // setAcceptDrops(true);

    // tell the KMainWindow that this is indeed the main widget
    setCentralWidget(m_view);

    // then, setup our actions
    setupActions();

    // and a status bar
    statusBar()->show();

    // apply the saved mainwindow settings, if any, and ask the mainwindow
    // to automatically save settings if changed: window size, toolbar
    // position, icon size, etc.
    setAutoSaveSettings();

    // allow the view to change the statusbar and caption
    connect(m_view, SIGNAL(signalChangeStatusbar(const QString&)),
            this,   SLOT(changeStatusbar(const QString&)));
    connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
            this,   SLOT(changeCaption(const QString&)));
	Varibles::LoadConfig () ;
}

Katalogue::~Katalogue()
{
	Varibles::SaveConfig () ;
}

void Katalogue::load(const KURL& url)
{
    QString target;
    // the below code is what you should normally do.  in this
    // example case, we want the url to our own.  you probably
    // want to use this code instead for your app

    #if 0
    // download the contents
    if (KIO::NetAccess::download(url, target))
    {
        // set our caption
        setCaption(url);

        // load in the file (target is always local)
        loadFile(target);

        // and remove the temp file
        KIO::NetAccess::removeTempFile(target);
    }
    #endif

    setCaption(url.url());
    m_view->openURL(url);
}

void Katalogue::setupActions()
{
	KStdAction::openNew(this, SLOT(fileNew()), actionCollection());
	KStdAction::open(this, SLOT(fileOpen()), actionCollection());
	KStdAction::save(this, SLOT(fileSave()), actionCollection());
	KStdAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
//	KStdAction::print(this, SLOT(filePrint()), actionCollection());
	KStdAction::quit(this, SLOT(fileQuit()), actionCollection());

	m_toolbarAction = KStdAction::showToolbar(this, SLOT(optionsShowToolbar()), actionCollection());
	m_statusbarAction = KStdAction::showStatusbar(this, SLOT(optionsShowStatusbar()), actionCollection());

	KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
	KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
	KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());

    // this doesn't do anything useful.  it's just here to illustrate
    // how to insert a custom menu and menu item
/*    KAction *custom = new KAction(i18n("Cus&tom Menuitem"), 0,
                                  this, SLOT(optionsPreferences()),
                                  actionCollection(), "custom_action");
*/
	KIconLoader * kil = KGlobal::iconLoader () ;

	scanDir = new KAction (i18n ("Scan &Directory"), kil->loadIconSet ("folder_new", KIcon::Toolbar /*KIcon::MainToolbar*//*,  KIcon::SizeSmallMedium, false*/), 0, this, SLOT (slotScanDir ()), actionCollection (), "scan_dir") ;
	addGroup = new KAction (i18n ("Add Disk &Group"), kil->loadIconSet ("wizard", KIcon::Toolbar/*, KIcon::SizeSmallMedium, false*/), 0, this, SLOT (slotAddGroup ()), actionCollection (), "add_group") ;

	findFile = KStdAction::find (this, SLOT (slotFindFile ()), actionCollection ()) ;

//	createGUI ("../share/apps/katalogue/katalogueui.rc") ;
//	createGUI ("/home/users/kasprzol/c/kde/katalogue/src/katalogueui.rc") ;
	createGUI () ;
}

void Katalogue::saveProperties(KConfig *config)
{
    // the 'config' object points to the session managed
    // config file.  anything you write here will be available
    // later when this app is restored

    if (m_view->currentURL() != QString::null)
        config->writeEntry("lastURL", m_view->currentURL());
}

void Katalogue::readProperties(KConfig *config)
{
    // the 'config' object points to the session managed
    // config file.  this function is automatically called whenever
    // the app is being restored.  read in here whatever you wrote
    // in 'saveProperties'

    QString url = config->readEntry("lastURL");

    if (url != QString::null)
        m_view->openURL(KURL(url));
}

void Katalogue::dragEnterEvent(QDragEnterEvent *event)
{
    // accept uri drops only
    event->accept(QUriDrag::canDecode(event));
}

void Katalogue::dropEvent(QDropEvent *event)
{
    // this is a very simplistic implementation of a drop event.  we
    // will only accept a dropped URL.  the Qt dnd code can do *much*
    // much more, so please read the docs there
    QStrList uri;

    // see if we can decode a URI.. if not, just ignore it
    if (QUriDrag::decode(event, uri))
    {
        // okay, we have a URI.. process it
        QString url, target;
        url = uri.first();

        // load in the file
        load(KURL(url));
    }
}

void Katalogue::fileNew()
{
    // this slot is called whenever the File->New menu is selected,
    // the New shortcut is pressed (usually CTRL+N) or the New toolbar
    // button is clicked

    // create a new window
//    (new Katalogue)->show();
	if (queryClose () == true) {
		/* nie mona tego stosowa, poniewa trzeba by przekaza do KatalogueView adres nowego
		   obiektu MyDocument
		delete m_mydoc ;
		m_mydoc = new MyDocument ;
		*/
		m_mydoc->newDoc () ;
		m_view->update () ;
	}
}

void Katalogue::fileOpen()
{
    // this slot is called whenever the File->Open menu is selected,
    // the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
    // button is clicked
	if (queryClose () == false) {
		return ;
	}
//    KURL url = KURLRequesterDlg::getURL(QString::null, this, i18n("Open Location") );
	KURL url = KFileDialog::getOpenURL(QString::null, i18n("*|All files"), this, i18n("Open File..."));
	if (!url.isEmpty()) {
		try {
			m_view->openURL(url) ;
		}
		catch (MyDirInfoStructures::FileReadException e) {
			KMessageBox::sorry (this, i18n ("Couldn't load the file")) ;
		}
	}
}

void Katalogue::fileSave()
{
    // this slot is called whenever the File->Save menu is selected,
    // the Save shortcut is pressed (usually CTRL+S) or the Save toolbar
    // button is clicked

    // save the current file
    if (m_mydoc->fileName () == "") {
    	fileSaveAs () ;
    } else 
    m_mydoc->save () ;
}

void Katalogue::fileSaveAs()
{
    // this slot is called whenever the File->Save As menu is selected,
    KURL file_url = KFileDialog::getSaveURL();
    if (!file_url.isEmpty() && file_url.isValid())
    {
        // save your info, here
	m_mydoc->saveAs (file_url.path ().local8Bit ().operator const char * ()) ;

    }
}

void Katalogue::filePrint()
{
/*
    // this slot is called whenever the File->Print menu is selected,
    // the Print shortcut is pressed (usually CTRL+P) or the Print toolbar
    // button is clicked
    if (!m_printer) m_printer = new KPrinter;
    if (m_printer->setup(this))
    {
        // setup the printer.  with Qt, you always "print" to a
        // QPainter.. whether the output medium is a pixmap, a screen,
        // or paper
        QPainter p;
        p.begin(m_printer);

        // we let our view do the actual printing
        QPaintDeviceMetrics metrics(m_printer);
        m_view->print(&p, metrics.height(), metrics.width());

        // and send the result to the printer
        p.end();
    }
*/
}

void Katalogue::fileQuit ()
{
	if (queryClose () == true) {
		KApplication::kApplication()->quit () ;
	}
}

void Katalogue::optionsShowToolbar()
{
    // this is all very cut and paste code for showing/hiding the
    // toolbar
    if (m_toolbarAction->isChecked())
        toolBar()->show();
    else
        toolBar()->hide();
}

void Katalogue::optionsShowStatusbar()
{
    // this is all very cut and paste code for showing/hiding the
    // statusbar
    if (m_statusbarAction->isChecked())
        statusBar()->show();
    else
        statusBar()->hide();
}

void Katalogue::optionsConfigureKeys()
{
    KKeyDialog::configure (actionCollection(), this);
}

void Katalogue::optionsConfigureToolbars()
{
    // use the standard toolbar editor
    saveMainWindowSettings( KGlobal::config(), autoSaveGroup() );
    KEditToolbar dlg(actionCollection());
    connect(&dlg, SIGNAL(newToolbarConfig()), this, SLOT(newToolbarConfig()));
    dlg.exec();
}

void Katalogue::newToolbarConfig()
{
    // this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
    // recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
    createGUI();
    applyMainWindowSettings( KGlobal::config(), autoSaveGroup() );
}

void Katalogue::optionsPreferences()
{
    // popup some sort of preference dialog, here
    KataloguePreferences dlg;
    if (dlg.exec())
    {
        // redo your settings
    }
}

void Katalogue::changeStatusbar(const QString& text)
{
    // display the text on the statusbar
    statusBar()->message(text);
}

void Katalogue::changeCaption(const QString& text)
{
    // display the text on the caption
    setCaption(text);
}

/** No descriptions */
void Katalogue::slotScanDir()
{
	ScanDirDialog sdd (this, m_mydoc->katalog ()) ;
	int rc = sdd.exec () ;
	if (rc == KDialogBase::Cancel) return ;
	QString qtdir = sdd.path () ;
	if (qtdir.isEmpty ()) return ;
	std::string dir = qtdir.local8Bit ().operator const char* () ;
	BaseClass * bc = sdd.group () ;
	changeStatusbar (i18n ("Scaning directory ") + dir ) ;
	std::cerr << i18n ("Scaning directory...") << dir << "\n" ;
	MyTopLevelDirInfo * mdi = new MyTopLevelDirInfo (dir, bc) ;
	if (sdd.name () != "") {
		mdi->name = sdd.name ().local8Bit ().operator const char* () ;
		// FIXME: doda pole z komentarzem do dialogu
		mdi->comment = sdd.name ().local8Bit ().operator const char* () ;
	}
	std::cerr << i18n ("Done scaning\n") ;
	if (bc->type == BaseClass::Katalog) {
		static_cast<MyKatalog*>(bc)->Disks.push_back (mdi) ;
	} else { // if (bc->type = DiskGroup)
		static_cast<MyDiskGroup*>(bc)->Disks.push_back (mdi) ;
	}
	m_mydoc->setModified (true) ;
	m_view->update () ;
	changeStatusbar ( i18n ("Ready") ) ;
}

/** No descriptions */
void Katalogue::slotAddGroup ()
{
	QString group = KInputDialog::getText (i18n ("Add group"), i18n ("Enter the name of the new group.")) ;
	if (!group.isEmpty ()) {
//		cerr << "dodawanie grupy o nazwie: " << group << "\n" ;
		MyDiskGroup * mdg = new MyDiskGroup (m_mydoc->katalog ()) ;
		mdg->name = group.local8Bit () ;
		m_mydoc->katalog()->DiskGroups.push_back (mdg) ;
		m_mydoc->setModified (true) ;
		m_view->update () ;
	}
}

/*!
    \fn Katalogue::slotFindFile ()
 */
void Katalogue::slotFindFile ()
{
	FindFileDialog ffd (this, m_mydoc->katalog ()) ;
	/*int rc = */ ffd.exec () ;
}

bool Katalogue::queryClose()
{
	bool completed = true ;
	
	if (m_mydoc->modified ()) {
		int want_save = KMessageBox::warningYesNoCancel(this,
                                         i18n("The current file has been modified.\n"
                                              "Do you want to save it?"),
                                         i18n("Warning")) ;
		switch(want_save) {
		case KMessageBox::Yes:
			fileSave () ; // fileSave () wywoa w razie potrzeby fileSaveAs ()
			if (m_mydoc->modified ()) {
				completed = false ;
/*			} else {
				completed = true ;
*/			}
			break ;
/*		case KMessageBox::No:
			completed = true ;
			break ;
*/		case KMessageBox::Cancel:
			completed = false ;
			break ;
/*		default:
			completed = false ;
			break;
*/		}
	}
	return completed ;
}

// #include "katalogue.moc"
