/***************************************************************************
 *   Copyright (C) 2004 by Maciej "Kasprzol" Kasprzyk                      *
 *   katalogue@katalogue.szluug.org                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KATALOGUE_H_
#define _KATALOGUE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kmainwindow.h>
// 
class KToggleAction;
class KURL;
class MyDocument ;
class KatalogueView ;

/**
 * @short Application Main Window
 * @author Maciej "Kasprzol" Kasprzyk <katalogue@katalogue.szluug.org>
 * @version 0.3
 */
class Katalogue : public KMainWindow
{
    Q_OBJECT
public:
    /**
     * Default Constructor
     */
    Katalogue();

    /**
     * Default Destructor
     */
    virtual ~Katalogue();

    /**
     * Use this method to load whatever file/URL you have
     */
    void load(const KURL& url);

protected:
    /**
     * Overridden virtuals for Qt drag 'n drop (XDND)
     */
    virtual void dragEnterEvent(QDragEnterEvent *event);
    virtual void dropEvent(QDropEvent *event);

protected:
    /**
     * This function is called when it is time for the app to save its
     * properties for session management purposes.
     */
    void saveProperties(KConfig *);

    /**
     * This function is called when this app is restored.  The KConfig
     * object points to the session management config file that was saved
     * with @ref saveProperties
     */
    void readProperties(KConfig *);
    
    bool queryClose () ;

private slots:
	void fileNew () ;
	void fileOpen () ;
	void fileSave () ;
	void fileSaveAs () ;
	void filePrint () ;
	void fileQuit () ;
	void optionsShowToolbar () ;
	void optionsShowStatusbar () ;
	void optionsConfigureKeys () ;
	void optionsConfigureToolbars () ;
	void optionsPreferences () ;
	void newToolbarConfig () ;

	void changeStatusbar(const QString& text);
	void changeCaption(const QString& text);
    
	void slotScanDir () ;
	void slotAddGroup () ;
	void slotFindFile ();

private:
    void setupAccel();
    void setupActions();

private:
    MyDocument * m_mydoc ;
    KatalogueView *m_view;

//    KPrinter   *m_printer;
    KToggleAction *m_toolbarAction;
    KToggleAction *m_statusbarAction;
    KAction * scanDir ;
    KAction * addGroup ;
    KAction * findFile ;
};
 

#endif // _KATALOGUE_H_
