/***************************************************************************
 *   Copyright (C) 2004 by Maciej "Kasprzol" Kasprzyk                      *
 *   katalogue@katalogue.szluug.org                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "katalogueview.h"
#include "treeview.h"
#include "mydirinfo.h"
#include "myplugin.h"
#include "mydocument.h"

#include <algorithm> // find
//#include <functional> // bind2nd
#include <assert.h>

#include <qpainter.h>
#include <qlayout.h> // QGridLayout
#include <qlabel.h>
#include <qtextedit.h>
#include <qheader.h>
#include <qsplitter.h>
#include <qpopupmenu.h>
//#include <qcstring.h>

#include <kurl.h>

//#include <ktrader.h>
//#include <klibloader.h>
#include <kmessagebox.h>
#include <krun.h>
#include <klocale.h>

KatalogueView::KatalogueView(QWidget *parent, MyDocument * mydoc)
    : QWidget(parent),
//      DCOPObject("KatalogueIface"),
      m_parent (parent),
      m_comment_layout (NULL),
      m_mydoc (mydoc),
      m_last_plugin (NULL)
{
   _vsplitter = new QSplitter ( Qt::Horizontal, this );

  TreeView::initIconCache () ;
  
  treeview = new TreeView ( _vsplitter, "treeview" );
  treeview->addColumn ( "Name" );
  treeview->setColumnWidthMode (0, QListView::Maximum) ;
  treeview->header ()->hide () ;
  treeview->setRootIsDecorated (true) ;
  treeview->setDefaultRenameAction (QListView::Accept) ;

  _hsplitter = new QSplitter ( Qt::Vertical, _vsplitter ) ;

  fileview = new TreeView ( _hsplitter, "fileview" ) ;
  fileview->addColumn ( i18n ("Name") ) ;
  fileview->addColumn ( i18n ("Size") ) ;
  fileview->addColumn ( i18n ("Date") ) ;
  fileview->addColumn ( i18n ("Mode") ) ;
  fileview->addColumn ( "uid/gid" ) ;
  fileview->addColumn ( i18n ("type") ) ;
  fileview->setAllColumnsShowFocus (true) ;
//  fileview->setSorting (-1) ;
  fileview->setShowToolTips (true) ;
  fileview->setShowSortIndicator (true) ;

  plugin_widget = new QWidget ( _hsplitter ) ;
  
  MyPlugin::initPlugins () ;
  
    connect ( treeview, SIGNAL( TreeItemSelected( const TreeItem & ) ),
	     this, SLOT ( treeItemSelected( const TreeItem & ) ) );
    connect ( fileview, SIGNAL( TreeItemSelected( const TreeItem & ) ),
	     this, SLOT ( treeItemSelected( const TreeItem & ) ) );
    connect ( /*static_cast<QListView*>*/(treeview), SIGNAL( contextMenuRequested( QListViewItem *, const QPoint& , int ) ),
	     this, SLOT( slotRightPressed( QListViewItem *, const QPoint &, int ) ) );
    connect ( /*static_cast<QListView*>*/(treeview), SIGNAL( itemRenamed( TreeItem *, const QString & ) ),
	     this, SLOT( slotItemRenamed (TreeItem *, const QString & ) ) ) ;
    addDirToTreeView (m_mydoc->katalog (), NULL) ;
}

KatalogueView::~KatalogueView()
{
	TreeView::deInitIconCache () ;
	if (m_last_plugin) m_last_plugin->destroyWidgets () ;
	MyPlugin::deInitPlugins () ;
}

/** No descriptions */
void KatalogueView::slotRightPressed( QListViewItem * item, const QPoint & pos, int col )
{
    if ( !item ) { // right pressed on viewport
/*	QPopupMenu menu( this );

	menu.insertItem( "&Large view", this, SLOT( viewLarge() ) );
	menu.insertItem( "&Small view", this, SLOT( viewSmall() ) );
	menu.insertSeparator();
	menu.insertItem( "Text at the &bottom", this, SLOT( viewBottom() ) );
	menu.insertItem( "Text at the &right", this, SLOT( viewRight() ) );
	menu.insertSeparator();
	menu.insertItem( "Arrange l&eft to right", this, SLOT( flowEast() ) );
	menu.insertItem( "Arrange t&op to bottom", this, SLOT( flowSouth() ) );
	menu.insertSeparator();
	menu.insertItem( "&Truncate item text", this, SLOT( itemTextTruncate() ) );
	menu.insertItem( "&Wordwrap item text", this, SLOT( itemTextWordWrap() ) );
	menu.insertSeparator();
	menu.insertItem( "Arrange items in &grid", this, SLOT( arrangeItemsInGrid() ) );
	menu.insertSeparator();
	menu.insertItem( "Sort &ascending", this, SLOT( sortAscending() ) );
	menu.insertItem( "Sort &descending", this, SLOT( sortDescending() ) );

	menu.setMouseTracking( TRUE );
	menu.exec( pos );
*/    } else { // on item
	QPopupMenu menu( this );

	int RENAME_ITEM = menu.insertItem( i18n ("Rename Item") );
	int REMOVE_ITEM = menu.insertItem( i18n ("Remove Item") );

	menu.setMouseTracking( TRUE );
	int id = menu.exec( pos );

	if ( id == -1 )
	    return;

	if ( id == RENAME_ITEM && item->renameEnabled(col) ) {
	    item->startRename(col) ;
	} else if ( id == REMOVE_ITEM ) {
		BaseClass * bc = const_cast<BaseClass*>( static_cast<TreeItem*>(item)->getItem () ) ;
		if (bc->type == BaseClass::TopLevelDir) {
			MyDirInfo * mdi = static_cast<const MyDirInfo *>(bc) ;
			std::vector<MyTopLevelDirInfo*> * vmdi ;
			if (mdi->parent ()->type == BaseClass::Katalog) {
				vmdi = & static_cast<MyKatalog*>(mdi->parent ())->Disks ;
			} else if (mdi->parent ()->type == BaseClass::DiskGroup) {
				vmdi = & static_cast<MyDiskGroup*>(mdi->parent ())->Disks ;
			}
			vmdi->erase (std::find (vmdi->begin (), vmdi->end (), mdi)) ;
			delete mdi ;
/*			vector<MyDirInfo*>::iterator iter ;
			for (iter = vmdi->begin () ; iter != vmdi->end () ; iter++) {
				if (*iter == mdi) {
					vmdi->erase (iter) ;
					break ;
				}
			}
*/		} else if (bc->type == BaseClass::DiskGroup) {
			MyDiskGroup * mdg = static_cast<MyDiskGroup*>(bc) ;
			std::vector<MyDiskGroup*> * vmdg = & static_cast<MyKatalog*>(mdg->parent ())->DiskGroups ;
			vmdg->erase (std::find (vmdg->begin (), vmdg->end (), mdg)) ;
			delete mdg ;
/*			vector<MyDiskGroup*>::iterator iter ;
			for (iter = vmdg->begin () ; iter != vmdg->end () ; iter++) {
				if (*iter == mdg) {
					vmdg->erase (iter) ;
					break ;
				}
			}
*/		} else {
			return ;
		}
// usu item z drzewa katalogw
		delete item ;
		m_mydoc->modified (true) ;
		update () ;
	}
    }
}

void KatalogueView::addDirToTreeView (BaseClass * p, TreeItem * parent)
{
	TreeItem * nowy ;
	if (parent) // TreeItem'y s niszczone przez list podczas zmiany zawartoci
		nowy = new TreeItem (parent, QString::fromLocal8Bit (p->name.c_str ()), p) ;
	else
		nowy = new TreeItem (treeview, QString::fromLocal8Bit (p->name.c_str ()), p) ;
	nowy->setOpen (false) ;
	if (p->type == BaseClass::Katalog) {
		for (uint i = 0 ; i < static_cast<MyKatalog*>(p)->DiskGroups.size () ; ++i) {
			addDirToTreeView (static_cast<MyKatalog*>(p)->DiskGroups[i], nowy) ;
		}
		for (uint i = 0 ; i < static_cast<MyKatalog*>(p)->Disks.size () ; ++i) {
			addDirToTreeView (static_cast<MyKatalog*>(p)->Disks[i], nowy) ;
		}
	} else if (p->type == BaseClass::DiskGroup) {
		for (unsigned int i = 0 ; i < static_cast<MyDiskGroup*>(p)->Disks.size () ; ++i) {
			addDirToTreeView (static_cast<MyDiskGroup*>(p)->Disks[i], nowy) ;
		}
	} else if (p->type >= BaseClass::Dir && p->type < BaseClass::DiskGroup) {
		for (unsigned int i = 0 ; i < static_cast<MyDirInfo*>(p)->subdirs.size () ; ++i) {
			addDirToTreeView (static_cast<MyDirInfo*>(p)->subdirs[i], nowy) ;
		}
	}
}

/** Change the directory displayed in fileview */
void KatalogueView::treeItemSelected ( const TreeItem & t)
{
	BaseClass * bc = t.getItem () ;
	if (bc->type >= BaseClass::Dir) {
		fileview->clear () ;
	}
	clearPluginPanel () ;
	
	if (bc->type == BaseClass::TopLevelDir || bc->type == BaseClass::DiskGroup) {
		// wywietl komentarz
		/*
		1. Utwrz layout (powinien wystarczy najprosszy verticalLauot
		2. Dodaj labela i editbox
		*/
		m_commented_item = bc ;
		m_comment_layout = new QVBoxLayout (plugin_widget, 2) ;
		m_comment_label = new QLabel (i18n ("Comment for <b>") + bc->name.c_str () + "</b>", plugin_widget, "comment_label") ;
		m_comment_textedit = new QTextEdit (plugin_widget, "comment_textedit") ;
		m_comment_textedit->setTextFormat (Qt::PlainText) ;
		if (m_commented_item->type == BaseClass::TopLevelDir ) {
//			std::cerr << "ustawiam komentarz: '" << static_cast<MyTopLevelDirInfo*>(m_commented_item)->comment.c_str () << "'" << std::endl ;
			m_comment_textedit->setText ( QString::fromLocal8Bit (static_cast<MyTopLevelDirInfo*>(m_commented_item)->comment.c_str ()) ) ;
		} else if (m_commented_item->type == BaseClass::DiskGroup ) {
//			std::cerr << "ustawiam komentarz: '" << static_cast<MyDiskGroup*>(m_commented_item)->comment.c_str () << "'" << std::endl ;
			m_comment_textedit->setText ( QString::fromLocal8Bit (static_cast<MyDiskGroup*>(m_commented_item)->comment.c_str ()) ) ;
		} else {
			assert (0 && m_commented_item->type) ;
		}
//		m_comment_textedit->setText (bc->name.c_str ()) ;
		m_comment_layout->addWidget (m_comment_label) ;
		m_comment_layout->addWidget (m_comment_textedit) ;
		m_comment_label->show () ;
		m_comment_textedit->show () ;
		connect ( m_comment_textedit, SIGNAL ( textChanged () ), this, SLOT ( slotCommentChanged() ) ) ;
	}
	
	switch (bc->type) {
	case BaseClass::TopLevelDir:
	case BaseClass::CompressedDir:
	case BaseClass::LinkToDir:
	case BaseClass::Dir: {
		MyDirInfo * m = static_cast<MyDirInfo*>(bc) ;
		treeview->setDir (m) ;
		for ( uint i = 0 ; i < m->subdirs.size () ; i++ ) {
        		(void) new TreeItem ( fileview, m->subdirs[i] );
		}
		for ( uint i = 0 ; i < m->files.size () ; i++ ) {
        		(void) new TreeItem ( fileview, m->files[i] ) ;
		}
		break ;
	}
	case BaseClass::DiskGroup: {
		MyDiskGroup * m = static_cast<MyDiskGroup*>(bc) ;
		treeview->setDir (m) ;
		for ( uint i = 0 ; i < m->Disks.size () ; i++ ) {
        		(void) new TreeItem ( fileview, m->Disks[i] ) ;
		}
		break ;
	}
	case BaseClass::Katalog: {
		MyKatalog * m = static_cast<MyKatalog*>(bc) ;
		for ( uint i = 0 ; i < m->DiskGroups.size () ; i++ ) {
        		(void) new TreeItem ( fileview, m->DiskGroups[i] ) ;
		}
		for ( uint i = 0 ; i < m->Disks.size () ; i++ ) {
        		(void) new TreeItem ( fileview, m->Disks[i] ) ;
		}
		break ;
	}
	case BaseClass::File:
	case BaseClass::LinkToFile: {
//		std::cerr << "Wybrae plik!\n" ;
//		FIXME: uruchomienie pliku
		MyFileInfo * m = static_cast<MyFileInfo*>(t.getItem ()) ;
		
		if (m->plugins.size ()) { // FIXME: narazie wywietlanie tylko jednego plugina, doda opcj wyboru plugina
			MyDirInfoStructures::MyFileInfoPluginInfo p = m->plugins[0] ;
			p.plugin->displayFileInfo (plugin_widget, p.PluginData, p.size) ;
			m_last_plugin = m->plugins[0].plugin ; //ostatnio wybrany plugin
		} /* else {
		} */
		break ;
	}
	} // switch (t.getItem ()->type) {
}

void KatalogueView::resizeEvent (QResizeEvent * e) 
{
      QWidget::resizeEvent (e) ;
      _vsplitter->resize (e->size () ) ;
}

void KatalogueView::update()
{
	treeview->clear () ;
	clearPluginPanel () ;
	addDirToTreeView(m_mydoc->katalog (), NULL) ;
}

void KatalogueView::slotItemRenamed (TreeItem * item, const QString & text)
{
	const char * bla = text ; // FIXME: "sprytna" konersja z QString'a na string'a
	static_cast<TreeItem*>(item)->getItem ()->name = bla ;
	m_mydoc->setModified (true) ;
}

void KatalogueView::slotCommentChanged ()
{
	QCString cstr = m_comment_textedit->text ().local8Bit () ;
	std::string * p ;
	if (m_commented_item->type == BaseClass::DiskGroup) {
		p = &static_cast<MyDiskGroup*>(m_commented_item)->comment ;
	} else if (m_commented_item->type == BaseClass::TopLevelDir) {
		p = &static_cast<MyTopLevelDirInfo*>(m_commented_item)->comment ;
	} else {
		std::cerr << "m_commented_item->type = " << m_commented_item->type << std::endl ;
		std::cerr << "m_commented_item->name = '" << m_commented_item->name << "'" << std::endl ;
		assert (0 && m_commented_item->type) ;
	}
	*p = cstr ;
	m_mydoc->modified (true) ;
}

void KatalogueView::print(QPainter *p, int height, int width)
{
    // do the actual printing, here
    // p->drawText(etc..)
}

QString KatalogueView::currentURL()
{
//    return m_html->url().url();
	return m_mydoc->fileName ().c_str () ;
}

void KatalogueView::openURL(QString url)
{
    openURL(KURL(url));
}

void KatalogueView::openURL(const KURL& url)
{
//    m_html->openURL(url);
	const char * bla = url.path () ; // FIXME: "sprytna" konwersja z QString'a na string'a
	m_mydoc->load (bla) ;
	m_commented_item = NULL ;
	clearPluginPanel () ;
	update () ;
}

void KatalogueView::slotOnURL(const QString& url)
{
    emit signalChangeStatusbar(url);
}

void KatalogueView::slotSetTitle(const QString& title)
{
    emit signalChangeCaption(title);
}

/*!
    \fn KatalogueView::clearPluginPanel ()
 */
void KatalogueView::clearPluginPanel ()
{
	if (m_last_plugin) {
		m_last_plugin->destroyWidgets () ;
		m_last_plugin = NULL ;
	} else if (m_comment_layout) {
/* - robione w slocie
		if (m_commented_item != NULL) {
			QCString cstr = m_comment_textedit->text ().local8Bit () ;
			// const char * bla = m_comment_textedit->text () ;
			const char * bla = cstr ;
			//std::cerr << "Tekst komentarza: " << bla << std::endl ;
			std::string * p ;

			if (m_commented_item->type == BaseClass::DiskGroup) {
				p = &static_cast<MyDiskGroup*>(m_commented_item)->comment ;
			} else if (m_commented_item->type == BaseClass::TopLevelDir) {
				p = &static_cast<MyTopLevelDirInfo*>(m_commented_item)->comment ;
			} else {
				std::cerr << "m_commented_item->type = " << m_commented_item->type << std::endl ;
				std::cerr << "m_commented_item->name = '" << m_commented_item->name << "'" << std::endl ;
				assert (0 && m_commented_item->type) ;
			}
			if (bla != *p) {
				*p = bla ;
				m_mydoc->modified (true) ;
			}
		}
*/
		delete m_comment_label ;
		delete m_comment_textedit ;
		delete m_comment_layout ;
		m_comment_layout = NULL ;
		m_commented_item = NULL ;
	}
}

// #include "katalogueview.moc"
