/***************************************************************************
 *   Copyright (C) 2004 by Maciej "Kasprzol" Kasprzyk                      *
 *   katalogue@katalogue.szluug.org                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KATALOGUEVIEW_H
#define KATALOGUEVIEW_H

#include <qwidget.h>
//#include <kparts/part.h>
//#include <katalogueiface.h>

class QPainter;
class KURL;
class TreeView ;
class TreeItem ;
class QListViewItem ;
class QResizeEvent ;
class QSplitter ;
class QVBoxLayout ;
class QTextEdit ;
class QLabel ;

class BaseClass ;
class MyDocument ;
class MyPlugin ;

/**
 * This is the main view class for Katalogue.  Most of the non-menu,
 * non-toolbar, and non-statusbar (e.g., non frame) GUI code should go
 * here.
 *
 * This katalogue uses an HTML component as an example.
 *
 * @short Main view
 * @author Maciej "Kasprzol" Kasprzyk
 * @version 0.3
 */
class KatalogueView : public QWidget /*, public KatalogueIface*/
{
    Q_OBJECT
public:
	/**
	 * Default constructor
	 */
    KatalogueView(QWidget *parent, MyDocument * mydoc);

	/**
	 * Destructor
	 */
    virtual ~KatalogueView();

    /**
     * Random 'get' function
     */
    QString currentURL();

    /**
     * Random 'set' function accessed by DCOP
     */
    virtual void openURL(QString url);

    /**
     * Random 'set' function
     */
    virtual void openURL(const KURL& url);

    /**
     * Print this view to any medium -- paper or not
     */
    void print(QPainter *, int height, int width);

    void update();    
    void clearPluginPanel () ;
  
public slots: // Public slots
  /** No descriptions */
  void slotRightPressed( QListViewItem * item, const QPoint & pos, int col ) ;
  void treeItemSelected ( const TreeItem & ) ;
  void slotItemRenamed(TreeItem * item, const QString & text) ;
  void slotCommentChanged () ;

protected:
	void resizeEvent (QResizeEvent * e) ;
	TreeView    * fileview ;
	TreeView    * treeview ;
	QSplitter   * _vsplitter ;
	QSplitter   * _hsplitter ;
	QWidget     * m_parent ;
	QWidget     * plugin_widget ;
	QTextEdit   * m_comment_textedit ;
	QLabel      * m_comment_label ;
	QVBoxLayout * m_comment_layout ;
	BaseClass   * m_commented_item ;
	MyDocument  * m_mydoc ;
	MyPlugin    * m_last_plugin ;

    void addDirToTreeView (BaseClass * p, TreeItem * parent = NULL) ;

signals:
    /**
     * Use this signal to change the content of the statusbar
     */
    void signalChangeStatusbar(const QString& text);

    /**
     * Use this signal to change the content of the caption
     */
    void signalChangeCaption(const QString& text);

private slots:
    void slotOnURL(const QString& url);
    void slotSetTitle(const QString& title);

private:
//    KParts::ReadOnlyPart *m_html;
};


#endif
