/***************************************************************************
 *   Copyright (C) 2004 by Maciej "Kasprzol" Kasprzyk                      *
 *   katalogue@katalogue.szluug.org                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <string>
#include <iostream>
#include <map>
#include <utility> //pair
#include <functional> // bind2nd w LoadPluginsTable
#include <cctype> // tolower ()

#include <unistd.h>  // chdir (), readlink ()

#include "mydirinfo.h"
#include "myplugin.h"

const unsigned char END_KATALOG = 0xef ;
const unsigned char BEGIN_GROUP = 0xf0 ;
const unsigned char END_GROUP = 0xf1 ;
const unsigned char BEGIN_DIRECTORY = 0xf5 ;
const char END_NAME = 0xf6 ; // to akurat musi by zwyky char (bez unsigned) gdy jest wykorzystywany w getline (strumien, string, END_NAME)
const unsigned char END_DIRECTORY = 0xf7 ;
const unsigned char BEGIN_FILE = 0xf8 ;
const unsigned char END_FILE = 0xf9 ;
const unsigned char BEGIN_PLUGIN_TABLE = 0xfa ;
const unsigned char END_PLUGIN_TABLE = 0xfb ;
const unsigned char BEGIN_PLUGIN = 0xfc ;
const unsigned char END_PLUGIN = 0xfd ;

using std::cout ;
using std::cerr ;
using std::string ;
using std::vector ;
using std::ofstream ;
using std::ifstream ;

namespace MyDirInfoStructures {

struct header { // nagwek pliku
  char head[8] ;
  unsigned char filever ; // wersja programu i formatu pliku * 10
  unsigned char progver ;
  unsigned char reserved[3] ;
} ;

} // namespace MyDirInfoStructures
 
using namespace MyDirInfoStructures ;

ofstream & operator<< (ofstream & s, const MyDirInfoStructures::header & h)
{
	s << h.head ;
	s.put (h.filever) ;
	s.put (h.progver) ;
	s.put (h.reserved[0]) ;
	s.put (h.reserved[1]) ;
	s.put (h.reserved[2]) ;
	return s ;
}
ifstream & operator>> (ifstream & s, MyDirInfoStructures::header & h)
{
	s.read (h.head, 7) ; // sam napis "KATALOG", bez nulla
	h.filever = s.get () ;
	h.progver = s.get () ;
	h.reserved[0] = s.get () ;
	h.reserved[1] = s.get () ;
	h.reserved[2] = s.get () ;
	return s ;
}

/*****************************************************************************
 * maxymalnie pogite rozwizanie majce na celu korzystanie z operatora <<  *
 * w odniesie do inta, shorta itp przy uywaniu (o/i)fstream                 *
 *****************************************************************************/

#if 0
namespace przeciazone_operatory {

template <class TYPE> ofstream & operator<< (ofstream & s, TYPE i)
{
/*
cout << "&i = " << &i << endl ;
cout << "&(i+1) = " << &i + 1 << endl ;
cout.setf (ios::hex, ios::basefield) ;
cout.setf (ios::showbase) ;
*/
for (char * p = (char*)&i ; p != (char*)(&i + 1) ; p++) {
//    static_cast<ostream &>(s) << *p ;
	s.put (*p) ;
//    cout << "p = " << (void*)p << "\t*p = " << (unsigned int)((unsigned char)*p) << endl ;
}
return s ;
}

template <class TYPE> ifstream & operator>> (ifstream & s, TYPE & i)
{
/*
cout << "&i = " << &i << endl ;
cout << "&(i+1) = " << &i + 1 << endl ;
cout.setf (ios::hex, ios::basefield) ;
cout.setf (ios::showbase) ;
*/
for (char * p = (char*)&i ; p != (char*)(&i + 1) ; p++) {
//    static_cast<istream &>(s) >> *p ;
	s.get (*p) ;
//    cout << "p = " << (void*)p << "\t*p = " << (unsigned int)((unsigned char)*p) << endl ;
}
return s ;
}

} ; // namespace przeciazone_operatory
#endif //if 0

#define OPERATOR_LESS(TYPE) ofstream & operator<< (ofstream & s, TYPE i) \
{ \
for (char * p = (char*)&i ; p != (char*)(&i + 1) ; ++p) { \
	s.put (*p) ; \
} \
return s ; \
}

#define OPERATOR_MORE(TYPE) ifstream & operator>> (ifstream & s, TYPE & i) \
{ \
for (char * p = (char*)&i ; p != (char*)(&i + 1) ; ++p) { \
	s.get (*p) ; \
} \
return s ; \
}

OPERATOR_LESS (int)
OPERATOR_MORE (int)

OPERATOR_LESS (unsigned int)
OPERATOR_MORE (unsigned int)

OPERATOR_LESS (long)
OPERATOR_MORE (long)

OPERATOR_LESS (unsigned long long)
OPERATOR_MORE (unsigned long long)

OPERATOR_LESS (unsigned short)
OPERATOR_MORE (unsigned short)

//OPERATOR_LES (time_t) // time_t = long
//OPERATOR_MORE (time_t)

/********* koniec pogitych rozwiza ***************/

namespace {

struct is_equal_name : public std::binary_function<MyPlugin *, const string, bool> { // getPluginInfo - nie jest const metod
	bool operator () (MyPlugin * plug, const string name) const { return plug->getPluginInfo () == name ; }
} ;

struct is_equal_num : public std::binary_function<const MyPlugin *, int, bool> {
	bool operator () (const MyPlugin * plug, int num) const { return plug->number () == num ; }
} ;

} // namespace {

/*inline*/ string BaseClass::AbsKataloguePath () const
{
	if (m_parent && m_parent->type == Katalog) return m_parent->name + "/" + name ;
	if (m_parent) return m_parent->AbsKataloguePath () + "/" + name ;
	return "/" + name ;
}

MyFileInfo::MyFileInfo (const std::string & name, const struct stat * st, EType t, BaseClass * p) : BaseClass (name, t, p), atime (st->st_atime),
	mtime(st->st_mtime), ctime(st->st_ctime), uid(st->st_uid), gid(st->st_gid), mode(st->st_mode),
	size(st->st_size)
{
}

MyFileInfo::MyFileInfo (const std::string & name, const std::string & parentDir, const struct stat * st, EType t, BaseClass * p) : BaseClass (name, t, p), atime (st->st_atime),
	mtime(st->st_mtime), ctime(st->st_ctime), uid(st->st_uid), gid(st->st_gid), mode(st->st_mode),
	size(st->st_size)
{
	if (Rozszerzenia.size () && name.find (".") != std::string::npos) { // nie ma kropki, FIXME: pluginy obsugujce pliki bez rozszerze?
		PluginsScan (parentDir + "/" + name) ;
	}
}
MyFileInfo::MyFileInfo (const std::string & name, const std::string & parentDir, const struct stat * st, const std::string & link, BaseClass * p) : BaseClass (name, LinkToFile, p), linkto (link), atime (st->st_atime),
	mtime(st->st_mtime), ctime(st->st_ctime), uid(st->st_uid), gid(st->st_gid), mode(st->st_mode),
	size(st->st_size)
{
	if (Rozszerzenia.size () && name.find (".") != std::string::npos) { // nie ma kropki, FIXME: pluginy obsugujce pliki bez rozszerze?
		PluginsScan (parentDir + "/" + name) ;
	}
}

void MyFileInfo::PluginsScan (const std::string & filefullpath)
{
	// wyszykuje tak dugo plugina a znajdzie. Np. jeli nazwa pliku to "adsadsad.html.tar.gz" to bdzie po koleii sprawdza
	// czy jest plugin dla "html.tar.gz", "tar.gz" i wreszcie dla "gz". Jeli znajdzie jakiego plugina to ju nie sprawdza pozostaych
	/*
	Stroustrup: 17.4.1.6: "Jeli lower_bound(k) nie znajduje k, to przekae iterator do pierwszego elementu z kluczem wikszym ni
	k lub do end (), jeli nie ma elementw o wikszym kluczu. Ten sposb informowania o porace stosuje si take w operacjach
	uper_bound i equal_range."
	*/
	unsigned int pos = 0 ;
	string sub ;
	typedef std::multimap<string, MyPlugin*>::iterator MI ; // pair<string, MyPlugin*>*
	std::pair<MI, MI> eq_r ;
//	std::cerr << "Analizuj plik '" << name << "'\n" ;
#if 1
	while ( true ) {
		pos = name.find (".", pos) ; // FIXME: zrobi nie rozrnianie duych i maych liter(?)
		// tu wystarczy samo name (bo abspath moe zawiera katalogi zawierajce kropki co by zmylio algorytm).
		if (pos == std::string::npos) {
//			std::cerr << "wychodzimy... (" << name << ")\n" ;
			return ;
		}
		sub  = name.substr (++pos) ;
		for (string::iterator aaa = sub.begin () ; aaa != sub.end () ; ++aaa) {
			*aaa = tolower (*aaa) ;
		}
		eq_r = Rozszerzenia.equal_range (sub) ;
//		std::cerr << "(first == end) = " << (eq_r.first == Rozszerzenia.end ()) << " (sub = \"" << sub << "\")\n" ;
		if (eq_r.first != Rozszerzenia.end () && eq_r.first->first == sub) {
			break ;
		}
	}
#else
	do {
		pos = name.find (".", pos) ; // FIXME: zrobi nie rozrnianie duych i maych liter(?)
		// tu wystarczy samo name (bo abspath moe zawiera katalogi zawierajce kropki co by zmylio algorytm).
		if (pos == name.npos) {
			std::cerr << "wychodzimy... (" << name << ")\n" ;
			return ;
		}
		sub  = name.substr (++pos) ;
		eq_r = Rozszerzenia.equal_range (sub) ;
		std::cerr << "(first == end) = " << (eq_r.first == Rozszerzenia.end ()) << " (sub = \"" << sub << "\")\n" ;
	} while (eq_r.first != Rozszerzenia.end () && eq_r.first->first != sub) ;
#endif
/*
	std::cerr << "(eq_r.first == eq_r.second) = '" << (eq_r.first == eq_r.second) << "'\n" ;
	if (eq_r.first != eq_r.second) {
		 std::cerr << eq_r.first->first << / * "\t" << eq_r.second ->first << * / "\n" ;
	}
*/	for (MI i = eq_r.first ; i != eq_r.second ; ++i) { // i->second to wskanik na MyPlugin
		MyDirInfoStructures::MyFileInfoPluginInfo mfpi ;
		const char * p ;
		char dest ; // jeli plugin zwrci niezerow warto to nie mona modyfikowa zwracanej wartoci
		
		p = i->second->getFileInfo (filefullpath.c_str (), &mfpi.size, &dest) ;
		if (p /* != NULL */) {
			i->second->setActive (true) ; // oznacz danego plugina jako aktywnego
			if (dest) {
				mfpi.PluginData = new char[mfpi.size] ; // niszczone w ~MyDirInfo ()
				memcpy (const_cast<void*>(mfpi.PluginData), p, mfpi.size) ;
			} else {
				mfpi.PluginData = p ;
			}
			mfpi.plugin = i->second ;
			plugins.push_back (mfpi) ;
		}
//		std::cerr << "Skanuj plik \"" << name << "\" pluginem \"" << mfpi.plugin->getPluginInfo () << "\"\n" ;
	}
}

unsigned long long MyDiskGroup::size ()
{
	unsigned long long ret = 0 ;
	std::vector<MyTopLevelDirInfo*>::iterator i, e = Disks.end () ;
	for (i = Disks.begin () ; i != e ; ++i) {
		ret += (*i)->size ;
	}
	return ret ;
}

/********************************************************************
 *                          Implementacja MyDirInfo                 *
 ********************************************************************/

MyDirInfo::~MyDirInfo ()
{
    std::vector <MyDirInfo *>::iterator iter2, e2 = subdirs.end () ;
    for (iter2 = subdirs.begin () ; iter2 != e2 ; ++iter2) {
//	cout << "Niszcz podkatalog: " << name << " / " << (*iter2)->name << "\n" ;
	delete *iter2 ;
    }
    
    std::vector <MyFileInfo *>::iterator iter1, e1 = files.end () ;
    for (iter1 = files.begin () ; iter1 != e1 ; ++iter1) {
	vector<MyDirInfoStructures::MyFileInfoPluginInfo>::iterator vi, vie = (*iter1)->plugins.end () ;
	for (vi = (*iter1)->plugins.begin () ; vi != vie ; vi++) {
		delete [] (char*)vi->PluginData ; // (*vi).PluginData ; - pami bya alokowana jako new char[]
	}
	delete *iter1 ;
    }
}

void MyDirInfo::scan (const std::string & DirToScan)
{
	DIR * dir ;
	struct dirent * de ;
	struct stat st ;

	dir = opendir ( DirToScan.c_str () ) ;
	if (dir == NULL) {
		std::cerr << "Cannot open dir " << std::endl ;
		return ;
	}

	while ( (de = readdir (dir)) ) { // warning: suggest parentheses around assignment used as truth value
		if (strcmp (de->d_name, ".") == 0 || strcmp (de->d_name, "..") == 0) {
			continue ;
		}
		lstat (string (DirToScan + "/" + de->d_name).c_str (), &st) ;
		if (S_ISDIR (st.st_mode)) {
			if (S_ISLNK (st.st_mode)) {
				cerr << de->d_name << " to link do katalogu.\n" ;
			}
			MyDirInfo * mdi ;
			std::cout <<"entering directory " << DirToScan << "/" << de->d_name << std::endl ;
			mdi = new MyDirInfo (de->d_name, DirToScan, &st, this) ;
//	cerr << "wrcono z katalogu " << abspath << "/" << de->d_name << "\n" ;
			subdirs.push_back (mdi) ;
			size += mdi->size ;
		} else if (S_ISREG (st.st_mode)) {
			MyFileInfo * fi ;
			if (S_ISLNK (st.st_mode)) {
				cerr << de->d_name << " to link do pliku.\n" ;
				char buf[1000] ;
				int i = readlink (std::string (DirToScan + "/" + de->d_name).c_str (), buf, 999) ;
				if ( i == -1 ) {
					perror ("readlink:") ;
					continue ;
				} else {
					buf[i] = '\0' ;
					stat (string (DirToScan + "/" + de->d_name).c_str (), &st) ;
					fi = new MyFileInfo (std::string (de->d_name), DirToScan, &st, buf, this) ;
				}
			} else { // zwyky plik
				fi = new MyFileInfo (de->d_name, DirToScan, &st, File, this) ;
			}
			files.push_back (fi) ;
			size += st.st_size ;
		} else {
			cerr << "Nie znany typ obiektu '" << de->d_name << "\n" ;
		}
	}
	closedir (dir) ;
}

MyDirInfo::MyDirInfo (std::ifstream & ifs, BaseClass * parent) throw (FileReadException) : // konstruktor wczytujcy katalog z pliku
MyFileInfo (parent)
{
	int i = 0 ;

	std::getline (ifs, name, END_NAME) ;

	type = BaseClass::EType (ifs.get ()) ;
	if (type == LinkToDir) {
		getline (ifs, linkto, END_NAME) ;
	}
	ifs >> atime >> mtime >> ctime >> uid >> gid >> mode >> size ;

	// wczytuj pliki i podkatalogi
	while (i != END_DIRECTORY) {
		i = ifs.get () ;
		if (i == BEGIN_FILE) { // pliki
			MyFileInfo * mfi = new MyFileInfo (this) ;
			getline (ifs, mfi->name, END_NAME) ;
			mfi->type = BaseClass::EType (ifs.get ()) ;
			if (mfi->type == LinkToFile) {
				getline (ifs, mfi->linkto, END_NAME) ;
			}
			ifs >> mfi->atime >> mfi->mtime >> mfi->ctime >> mfi->uid >> mfi->gid >> mfi->mode >> mfi->size ;
			// pluginy
			while ((i = ifs.get ()) == BEGIN_PLUGIN) {
				MyDirInfoStructures::MyFileInfoPluginInfo mfipi ;
				vector<MyPlugin*>::const_iterator vci ;
				int num ;
				ifs >> num >> mfipi.size ;
				vci = find_if (Pluginy.begin (), Pluginy.end (), bind2nd (is_equal_num (), num)) ;
				if (vci != Pluginy.end ()) { // moe nie by plugina o tym numerze
					mfipi.plugin = *vci ;
					char * _bla = new char[mfipi.size] ; // zwalniane w ~MyDirInfo ()
					ifs.read (_bla, mfipi.size) ;
					// ifs.get () ; // END_PLUGIN - nie czyta tutaj - wczytywane pod koniec while
					mfipi.PluginData = _bla ;
					mfi->plugins.push_back (mfipi) ;
//					cerr << "Znaleniono plugin nr " << num << ".\n" ;
				} else { // nie ma tego plugina, wic omijamy ca sekcje z danymi plugina
					ifs.seekg (mfipi.size, std::ios_base::cur) ; // do przodu o mfipi.size bajtw
//					cerr << "Nie znaleziono plugina nr " << num << ".\n" ;
				}
				ifs.get () ; // END_PLUGIN
			}
			ifs.putback (i) ;
			if ((i = ifs.get ()) != END_FILE) {
				throw FileReadException (NoEndOfFile, i, ifs.tellg ()) ;
			}
			files.push_back (mfi) ;
		} else if (i == BEGIN_DIRECTORY) { // podkatalogi
			MyDirInfo * mdi = new MyDirInfo (ifs, this) ;
			subdirs.push_back (mdi) ;
		} else if (i == END_DIRECTORY) { // koniec katalogu - ta linia musi by aby nie wyskakiwa poniszy else dla i == END_DIRECTORY
		} else {
			throw FileReadException (UnknownChar, i, ifs.tellg ()) ;
		}
	} // while (i != END_DIRECTORY) {
}

void MyDirInfo::SaveDir (std::ofstream & of)
{
  of.put (BEGIN_DIRECTORY) ;
  of << name ;
  of.put (END_NAME) ;
  of.put (type) ;
  if (type == LinkToDir) {
    of << linkto ;
    of.put (END_NAME) ;
  }
  of << atime << mtime << ctime << uid << gid << mode << size ;
  for (uint i = 0 ; i < files.size () ; ++i) {
	of.put (BEGIN_FILE) ;
	of << files[i]->name ;
	of.put (END_NAME) ;
	of.put (files[i]->type) ;
	if (files[i]->type == LinkToFile) {
		of << files[i]->linkto ;
		of.put (END_NAME) ;
	}
	of << files[i]->atime << files[i]->mtime << files[i]->ctime << files[i]->uid << files[i]->gid << files[i]->mode << files[i]->size ;
	// pluginy
	vector<MyDirInfoStructures::MyFileInfoPluginInfo>::const_iterator vci, vcie = files[i]->plugins.end () ;
	for (vci = files[i]->plugins.begin () ; vci != vcie ; ++vci) {
		of.put (BEGIN_PLUGIN) ;
		of << vci->plugin->number () << vci->size ;
		of.write ((char*)vci->PluginData, vci->size) ;
		of.put (END_PLUGIN) ;
	}
        of.put (END_FILE) ;
  }
  for (uint i = 0 ; i < subdirs.size () ; ++i) {
          subdirs[i]->SaveDir (of) ;
  }
  of.put (END_DIRECTORY) ; // koniec dysku
}

/********************************************************************
 *                Implementacja MyTopLevelDirInfo                   *
 ********************************************************************/
MyTopLevelDirInfo::MyTopLevelDirInfo (const std::string & Directory, BaseClass * Up) :
MyDirInfo (Up)
{
	type = TopLevelDir ;
	// name = strrchr (Directory, '/') + 1 ;
	// name = abspath ().substr (abspath ().rfind ("/") + 1) ;
	name = Directory ;
	scan (Directory) ;
}

MyTopLevelDirInfo::MyTopLevelDirInfo (ifstream & ifs, BaseClass * parent) throw (FileReadException) : // konstruktor wczytujcy katalog z pliku
MyDirInfo (parent)
{
	int i = 0 ;

	std::getline (ifs, name, END_NAME) ;
	std::getline (ifs, comment, END_NAME) ;
	type = TopLevelDir ;

	/*
	type = BaseClass::EType (ifs.get ()) ;
	if (type != Dir) { // wczeniejsze wersje
		getline (ifs, comment, END_NAME) ;
	}
	*/
	ifs >> atime >> mtime >> ctime >> uid >> gid >> mode >> size ;

	// wczytuj pliki i podkatalogi
	while (i != END_DIRECTORY) {
		i = ifs.get () ;
		if (i == BEGIN_FILE) { // pliki
			MyFileInfo * mfi = new MyFileInfo (this) ;
			getline (ifs, mfi->name, END_NAME) ;
			mfi->type = BaseClass::EType (ifs.get ()) ;
			if (mfi->type == LinkToFile) {
				getline (ifs, mfi->linkto, END_NAME) ;
			}
			ifs >> mfi->atime >> mfi->mtime >> mfi->ctime >> mfi->uid >> mfi->gid >> mfi->mode >> mfi->size ;
			// pluginy
			while ((i = ifs.get ()) == BEGIN_PLUGIN) {
				MyDirInfoStructures::MyFileInfoPluginInfo mfipi ;
				vector<MyPlugin*>::const_iterator vci ;
				int num ;
				//cerr << "pozycja w pliku: " << ifs.tellg () << std::hex << "(0x" << ifs.tellg () << ")" << std::dec << std::endl ;
				ifs >> num >> mfipi.size ;
				//cerr << "Wczytany nr plugina: " << num << " rozmiar danych plugina: " << mfipi.size << " pozycja w pliku: " << ifs.tellg () << std::hex << "(0x" << ifs.tellg () << ")" << std::dec << std::endl ;
				vci = find_if (Pluginy.begin (), Pluginy.end (), bind2nd (is_equal_num (), num)) ;
				if (vci != Pluginy.end ()) { // moe nie by plugina o tym numerze
					mfipi.plugin = *vci ;
					char * _bla = new char[mfipi.size] ; // zwalniane w ~MyDirInfo ()
					ifs.read (_bla, mfipi.size) ;
					// ifs.get () ; // END_PLUGIN - nie czyta tutaj - wczytywane pod koniec while
					mfipi.PluginData = _bla ;
					mfi->plugins.push_back (mfipi) ;
//					cerr << "Znaleniono plugin nr " << num << ".\n" ;
				} else { // nie ma tego plugina, wic omijamy ca sekcje z danymi plugina
					ifs.seekg (mfipi.size, std::ios_base::cur) ; // do przodu o mfipi.size bajtw
//					cerr << "Nie znaleziono plugina nr " << num << ".\n" ;
				}
				ifs.get () ; // END_PLUGIN
			}
			ifs.putback (i) ;
			if ((i = ifs.get ()) != END_FILE) {
				throw FileReadException (NoEndOfFile, i, ifs.tellg ()) ;
			}
			files.push_back (mfi) ;
		} else if (i == BEGIN_DIRECTORY) { // podkatalogi
			MyDirInfo * mdi = new MyDirInfo (ifs, this) ;
			subdirs.push_back (mdi) ;
		} else if (i == END_DIRECTORY) { // koniec katalogu - ta linia musi by aby nie wyskakiwa poniszy else dla i == END_DIRECTORY
		} else {
			throw FileReadException (UnknownChar, i, ifs.tellg ()) ;
		}
	} // while (i != END_DIRECTORY) {
}

void MyTopLevelDirInfo::SaveDir (std::ofstream & of)
{
	of.put (BEGIN_DIRECTORY) ;
	of << name ;
	of.put (END_NAME) ;
	of << comment ;
	of.put (END_NAME) ;

	of << atime << mtime << ctime << uid << gid << mode << size ;
	for (uint i = 0 ; i < files.size () ; ++i) {
		of.put (BEGIN_FILE) ;
		of << files[i]->name ;
		of.put (END_NAME) ;
		of.put (files[i]->type) ;
		if (files[i]->type == LinkToFile) {
			of << files[i]->linkto ;
			of.put (END_NAME) ;
		}
		of << files[i]->atime << files[i]->mtime << files[i]->ctime << files[i]->uid << files[i]->gid << files[i]->mode << files[i]->size ;
		// pluginy
		vector<MyDirInfoStructures::MyFileInfoPluginInfo>::const_iterator vci, vcie = files[i]->plugins.end () ;
		for (vci = files[i]->plugins.begin () ; vci != vcie ; ++vci) {
			of.put (BEGIN_PLUGIN) ;
			of << vci->plugin->number () << vci->size ;
			of.write ((char*)vci->PluginData, vci->size) ;
			of.put (END_PLUGIN) ;
		}
        	of.put (END_FILE) ;
	}
	for (uint i = 0 ; i < subdirs.size () ; i++) {
		subdirs[i]->SaveDir (of) ;
	}
	of.put (END_DIRECTORY) ; // koniec dysku
}


/********************************************************************
 *                      Implementacja MyKatalog                     *
 ********************************************************************/


void MyKatalog::SaveDiskGroups (ofstream & of)
{
	for (uint i = 0 ; i < DiskGroups.size () ; ++i) {
		of.put (BEGIN_GROUP) ;
		of << DiskGroups[i]->name ;
		of.put (END_NAME) ;
		of << DiskGroups[i]->comment ;
//		cout << "Zapisuj komentarz grupy: '" << DiskGroups[i]->comment << "' (" << DiskGroups[i]->comment.size () << ")\n" ;
		of.put (END_NAME) ;
		for (uint j = 0 ; j < DiskGroups[i]-> Disks.size () ; ++j) {
			DiskGroups[i]->Disks[j]->SaveDir (of) ;
		}
		of.put (END_GROUP) ;
	}
}
void MyKatalog::LoadDiskGroups (ifstream & ifs) throw (FileReadException)
{
	int i ;

	while ((i = ifs.get ()) == BEGIN_GROUP) {
		MyDiskGroup * mdg = new MyDiskGroup (this) ;
		getline (ifs, mdg->name, END_NAME) ;
		getline (ifs, mdg->comment, END_NAME) ;

		while ( (i = ifs.get ()) == BEGIN_DIRECTORY) {
			MyTopLevelDirInfo * mdi = new MyTopLevelDirInfo (ifs, mdg) ;
			mdg->Disks.push_back (mdi) ;
		}
		if (i != END_GROUP) { // i zawiera juz warto wczytan przez powysze while
			throw FileReadException (NoEndOfGroup, i, ifs.tellg ()) ;
		}
		DiskGroups.push_back (mdg) ;
	}
	ifs.putback (i) ;
}
/*
void MyKatalog::SaveDisks (std::ofstream & of)
{
	for (uint i = 0 ; i < Disks.size () ; i++) {
		Disks[i]->SaveDir (of) ;
	}
}
*/
void MyKatalog::LoadDisks (std::ifstream & ifs)
{
	int i ;
	while ( (i = ifs.get ()) == BEGIN_DIRECTORY) {
		MyTopLevelDirInfo * mdi = new MyTopLevelDirInfo (ifs, this) ;
		Disks.push_back (mdi) ;
	}
	ifs.putback (i) ;
}

void MyKatalog::LoadPluginsTable (std::ifstream & ifs)
{
	// znajdz plugin na podstawie nazwy
	string nazwa ;
	vector<MyPlugin*> ::iterator vi ;
	int num, plugver, dataver, i ;
	while ((i = ifs.get ()) == BEGIN_PLUGIN_TABLE) {
		ifs >> num ;
		getline (ifs, nazwa, END_NAME) ;
		ifs >> plugver >> dataver ;
		ifs.get () ; // END_PLUGIN_TABLE
		vi = find_if (Pluginy.begin (), Pluginy.end (), bind2nd (is_equal_name (), nazwa)) ;
		if (vi == Pluginy.end ()) {
			std::cerr << "Nie znaleniono wymaganego plugina: " << nazwa << ". Przy zapisie pliku dane tego plugina zostan stracone.\n" ;
		} else {
			if ((*vi)->isCompatibleWith (plugver, dataver)) {
				(*vi)->setActive (true) ;
				(*vi)->setNumber (num) ;
			} else {
				std::cerr << "Znalezniono odpowiedni plugin, lecz nie obsuguje on tej wersji zapisanych danych. (" << nazwa << "). Przy zapisie pliku dane tego plugina zostan stracone.\n" ;
			}
		}
	}
	ifs.putback (i) ;
}

void MyKatalog::SavePluginsTable (std::ofstream & of)
{
	std::vector<MyPlugin*>::const_iterator i, e = Pluginy.end () ;
	int num = 0 ;
	for (i = Pluginy.begin () ; i != e ; ++i) {
		if ( (*i)->active ()) {
			of.put (BEGIN_PLUGIN_TABLE) ;
			(*i)->setNumber (num++) ;
			of << (*i)->number () ;
			of << (*i)->getPluginInfo () ;
			of.put (END_NAME) ;
			of << (*i)->getPluginVer () << (*i)->getPluginDataVer () ;
			of.put (END_PLUGIN_TABLE) ;
		}
	}
}

void MyKatalog::SaveToFile (const char * filename)
{
  MyDirInfoStructures::header h01 = { "KATALOG", 1, 1, { '\0','\0', '\0' } };
  std::ofstream of (filename) ;
  of <<  h01 ;
  of << name ;
  of.put (END_NAME) ;
  SavePluginsTable (of) ;
  SaveDiskGroups (of) ;
//  SaveDisks (of) ;
	for (uint i = 0 ; i < Disks.size () ; ++i) {
		Disks[i]->SaveDir (of) ;
	}


  of.put (END_KATALOG /*0xff*/) ;
}

void MyKatalog::LoadFromFile (const char * filename) throw (FileReadException)
{
	unsigned char i = 0 ;
	MyDirInfoStructures::header h01 ;

	destruct () ;

	std::ifstream ifs (filename) ;
	ifs >> h01 ;

	if (strncmp (h01.head, "KATALOG", 7) != 0 || strncmp ((const char*)h01.reserved, "\0\0\0", 3) != 0) {
		throw ( FileReadException (WrongHeader, i, 0) ) ;
	}
	if (h01.filever != 1 || h01.progver != 1) {
		throw ( FileReadException (WrongFileVersion, i, 0) ) ;
	}

	getline (ifs, name, END_NAME) ;

	try {
		while (i != END_KATALOG /*0xff*/) {
			i = ifs.peek () ;
			switch (i) {
			case BEGIN_PLUGIN_TABLE:
				LoadPluginsTable (ifs) ;
				break ;
			case BEGIN_GROUP:
				LoadDiskGroups (ifs) ;
				break ;
			case BEGIN_DIRECTORY:
				LoadDisks (ifs) ;
				break ;
			case END_KATALOG /*0xff*/:
				break ;
			default:
				throw ( FileReadException (UnknownChar, i, ifs.tellg ()) ) ;
			}
		}
	}
	catch (const FileReadException & fre) {
		cerr << "FileReadException.type = " << fre.type << std::endl ;
		if (fre.type == UnknownChar) {
			cerr << "nieznany znak \"" << fre.znak << "\"(" << int(fre.znak) << "[0x" << std::hex << int (fre.znak) << std::dec << "])\n" ;
		} else if (fre.type == NoEndOfFile) {
			cerr << "Brak koca pliku - \"" << fre.znak << "\"(" << int(fre.znak) << "[0x" << std::hex << int (fre.znak) << std::dec << "])\n" ;
		}

		cerr << "Pozycja na ktrej napotkano bd: " << fre.pos << std::hex << "(0x" << fre.pos << ")" << std::dec << std::endl ;
		ifs.close () ;
		throw ;
	}
}

MyKatalog & MyKatalog::operator = (const MyKatalog & rhs)
{
	destruct () ;
	name = rhs.name ;
	Disks = rhs.Disks ;
	DiskGroups = rhs.DiskGroups ;
	for (std::vector<MyDiskGroup*>::iterator it = DiskGroups.begin () ; it != DiskGroups.end () ; ++it) {
		(*it)->parent (this) ;
	}
	return *this ;
}
