/***************************************************************************
 *   Copyright (C) 2004 by Maciej "Kasprzol" Kasprzyk                      *
 *   katalogue@katalogue.szluug.org                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MYPLUGIN_H
#define MYPLUGIN_H

#include <string>
#include <map>
#include <vector>

class QWidget ;

/**
 * Obsuguje dziaania na pluginach. Kady plugin musi udostpnia nastpujce funkcje:
 * a) GetPluginInfo - zwraca const char * zawierajcy nazw plugina. Wersj plugina i wersj
 *    formatu danych zwraca poprzez przekazane wskaniki do int'a.
 *    gdy na podstawie tej wartoci program bdzie rozrnia pluginy i bdzie j zapisywa
 *    w pliku i uywa jej do znalezienia waniwego plugina.
 * b) IsCompatibleWith - pobiera dwa inty (wersj plugina i wersj danych) i okrela czy plugin jest
 *    z ni kompatybilny (czy obsuguje dane zapisane przez t wersj). Zwraca 1 (true) lub 0 (false).
 * c) GetFileType - zwraca const char * zawierajcy list obsugiwanych typw plikw. Lista ta
 *    jest w formacie "mp3;wav;tar.bz2;txt"
 * d) GetFileInfo - zwraca void * zawierajcy dane jakie plugin ma o danym pliku. Rozmiar tych danych
 *    zwraca w zmiennej przekazanej do niej przez wskanik. Ostatnia warto okrela czy program ma samemu
 *    zwolni pami zwrcon przez plugina (bdzie zwolniona jako delete [](char*)p)
 * e) SetParentWidget - ustawia widget ktry bdzie rodzicem wszystkich widgetw tworzonych przez
 *    plugina.
 * f) DisplayFileInfo - wywietla informacje o pliku przekaze funkcji jako parametr.
 * g) DestroyWidgets - niszczy wszystkie widgety stworzone podczas DisplayFileInfo.
 * 
 * WANE: zapoznaj si z C++dlopen (/usr/doc/Linux-mini-HOWTOs)
 **/
/**
@author Maciej "Kasprzol" Kasprzyk
*/
class MyPlugin
{
	typedef const char * (*get_file_info_func) (const char *, unsigned int *, char *) ;
	typedef const char * (*get_file_type_func) () ;
	typedef void (*display_file_info_func) (const QWidget *, const void *, unsigned int) ;
	typedef const char * (*get_plugin_info_func) (int *, int *) ;
	typedef void (*destroy_widgets_func) () ;
	typedef int /* bool */ (*is_compatible_with_func) (int, int) ; // ngby by zwracany bool, ale jeli plugin jest napisany w C, to tam nie ma bool
	std::string m_dll_name ;
	int m_plug_ver ;
	int m_data_ver ;
	void * m_dll_handler ;
	bool m_ok ;
	bool m_active ; // okrela czy plugin jest wykorzystywany. Uywane do zapisania tablicy aktywnych pluginw do pliku
	int m_num ; // numer - uywane podczas zapisywania i wczytywania plikw
	get_file_type_func m_get_file_type ;
	get_file_info_func m_get_file_info ;
	display_file_info_func m_display_file_info ;
	get_plugin_info_func m_get_plugin_info ;
	destroy_widgets_func m_destroy_widgets ;
	is_compatible_with_func m_is_compatible_with ;
	enum ETYPE { // typ informacji zawartej w pluginie:
		Text,
		Graphic, // (grafika (np. miniaturka)
		FileList // lista plikw (np. spakowany plik)
	} m_type ;
	int loadplug (const char *) ;
public:
	MyPlugin (const char * plugin_name) { loadplug (plugin_name) ; }
	MyPlugin (const std::string & plugin_name) { loadplug (plugin_name.c_str ()) ; }
	~MyPlugin () ;
	bool loaded () const{ return m_ok ; }
	int getPluginVer () const { return m_plug_ver ; } // zwraca wersj plugina
	int getPluginDataVer () const { return m_data_ver ; } // zwraca wersje formatu danych plugina (co jak filever z MyDirInfoStructures::header)
	bool active () const { return m_active ; }
	void setActive (bool a) { m_active = a ; }
	int number () const { return m_num ; }
	void setNumber (int n) { m_num = n ; }
    
  const char * getFileType () const { return (*m_get_file_type) () ; }
  const char * getFileInfo (const char * filename, unsigned int * size, char * dest) const { return (*m_get_file_info) (filename, size, dest) ; }
  void displayFileInfo (QWidget * widget, const void * filedata, unsigned int size) const { (*m_display_file_info) (widget, filedata, size) ; }
  const char * getPluginInfo () { return (*m_get_plugin_info) (&m_plug_ver, &m_data_ver) ; }
  void destroyWidgets () const { (*m_destroy_widgets) () ; }
  int isCompatibleWith (int ver, int dataver) const { return (*m_is_compatible_with) (ver, dataver) ; }
  
  static void initPlugins () ;
  static void deInitPlugins () ;
} ;

extern std::multimap<std::string, MyPlugin*> Rozszerzenia ;
extern std::vector<MyPlugin*> Pluginy ;

#endif
