/***************************************************************************
 *   Copyright (C) 2004 by Maciej "Kasprzol" Kasprzyk                      *
 *   katalogue@katalogue.szluug.org                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// include files for QT
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>
//#include <qtextcodec.h>
#include <qcombobox.h>

// include files for KDE
#include <kdirselectdialog.h>
#include <klocale.h>
//#include <kdialogbase.h> /* includowane przez scandirdialog.h */

// application specific includes
#include "scandirdialog.h"
#include "mydirinfo.h"

/*
from qt FAQ:
Do I have to delete my child widgets? 

Ordinarily no. Child widgets will be automatically destroyed by the parent widget
when that is destroyed. See the documentation of ~QObject() for details.
*/

ScanDirDialog::ScanDirDialog(QWidget *parent, MyKatalog * myk)
    : KDialogBase (KDialogBase::Plain, i18n ("Scan Directory"), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, parent, "scandir", true, true),
      m_mykatalog (myk)
{
	QFrame * page = plainPage () ;

	QGridLayout * grid = new QGridLayout ( page, 5, 2, 3 /* space beetwen elements in pixels */ );
	
	QLabel *label = new QLabel( i18n ("Directory containg files and subdirs to scan:"), page, "add_disk" ) ;
// Katalog zawierajcy dane do skanowania\nUWAGA: Program zakada e wprowadzisz sensowne dane (to znaczy poprawn ciek do katalogu a nie np. dat urodzenia). Jeli wprowadzone przez Ciebie dane nie bd poprawne i co zego si stanie to pretensje miej tylko do siebie."
	grid->addWidget (label, 0, 0 );
	
	m_le_scandir = new QLineEdit( page, "path_to_scan" ) ;
	m_le_scandir->setMinimumWidth (fontMetrics ().maxWidth () * 20) ;
//	label1->setMaximumWidth (m_lineedit->minimumWidth ()) ;
	grid->addWidget ( m_le_scandir, 1, 0 ) ;

	QPushButton * button = new QPushButton ( i18n ("Browse..."), page ) ;
	grid->addWidget ( button, 1, 1 ) ;

	label = new QLabel( i18n ("Use this name (leave blank for the name of the directory):"), page, "select_group_label" ) ;
	grid->addWidget (label, 2, 0 );

	m_le_name = new QLineEdit( page, "scaned_name" ) ;
	m_le_name->setMinimumWidth(fontMetrics ().maxWidth () * 20) ;
	grid->addWidget ( m_le_name, 3, 0 ) ;

	label = new QLabel( i18n ("Add this directory to group:"), page, "select_group_label" ) ;
	grid->addWidget (label, 4, 0 );
	m_combobox = new QComboBox( FALSE, page, "select-group-combobox" ) ;
	m_combobox->insertItem (QString (myk->name.c_str ())) ;
	for (uint i = 0 ; i < myk->DiskGroups.size () ; i++) {
		m_combobox->insertItem (QString (myk->DiskGroups[i]->name.c_str ())) ;
	}
	m_combobox->setCurrentItem (0) ;
	grid->addWidget (m_combobox, 4, 1) ;

	connect (button, SIGNAL ( clicked () ),
		 this, SLOT ( slotBrowseButton () ) ) ;
}

/** No descriptions */
void ScanDirDialog::slotBrowseButton()
{
	const char * path = getenv ("HOME") ;
	KURL url = KDirSelectDialog::selectDirectory (path) ;
	if (!url.isEmpty ()) {
		m_dir = url.path () ;
		m_le_scandir->setText (m_dir) ;
	}
	
}

BaseClass * ScanDirDialog::group ()
{
	if (comboindex == -1) return NULL ;
	if (comboindex == 0) {
		return (m_mykatalog) ;
	} else {
		return (m_mykatalog->DiskGroups[comboindex - 1]) ;
	}
}

/** No descriptions */
void ScanDirDialog::slotOk()
{
	m_dir = m_le_scandir->text () ;
	m_name = m_le_name->text () ;
	comboindex = m_combobox->currentItem () ;
	accept () ;
}

/** No descriptions */
void ScanDirDialog::slotCancel ()
{
	m_dir = "" ;
	m_name = "" ;
	comboindex = -1 ;
	reject () ;
}
